/**
 * @author diverKon <diverKon@gmail.com>
 * @license http://www.opensource.org/licenses/bsd-license.php
 */
package jp.ne.sakura.kkkon.yamahart;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;


/**
 *
 */
public class SSH
{
    private static final Logger logger = LoggerFactory.getLogger( Telnet.class );


    protected void setOption( final JSch client )
    {
        if ( null == client )
        {
            throw new IllegalArgumentException();
        }

        //client.setKnownHosts( "~/.ssh/known_hosts" );
        //JSch.setConfig( "", "" );
        JSch.setConfig( "StrictHostKeyChecking", "no" );
    }

    protected Session connect( final JSch client, final Account account )
    {
        if ( null == client || null == account )
        {
            throw new IllegalArgumentException();
        }

        Session session = null;
        try
        {
            session = client.getSession( account.username, account.hostname );
        }
        catch ( JSchException e )
        {
            e.printStackTrace( System.err );
        }

        if ( null != session )
        {
            try
            {
                Thread.sleep( 1*1000 );
            }
            catch ( InterruptedException e )
            {
                e.printStackTrace( System.err );
            }
        }

        return session;
    }

    protected void login( final Session session, final Account account, final ByteArrayOutputStream outStreamResult )
    {
        if ( null != session )
        {
            session.setPassword( account.passwordLogin );
            try
            {
                session.connect( 10 * 1000 );
            }
            catch ( JSchException e )
            {
                e.printStackTrace( System.err );
            }
            session.setPassword( "" );
        }
    }

    protected void administrator( final Session session, final Channel channel, final Account account, final ByteArrayOutputStream outStreamResult )
    {
        if ( null != session )
        {
            session.setPassword( account.getPasswordAdmin() );
        }

        if ( null != channel )
        {

            InputStream inStream = null;
            OutputStream outStream = null;

            try
            {
                inStream = channel.getInputStream();
                outStream = channel.getOutputStream();
            }
            catch ( IOException e )
            {
                e.printStackTrace( System.err );
            }

            boolean remoteEcho = false;

            remoteEcho = true;
            try
            {
                outStream.write( "administrator".getBytes() );
                outStream.write( "\r".getBytes() );
                outStream.flush();
            }
            catch ( IOException e )
            {
                e.printStackTrace( System.err );
            }

            {
                outStreamResult.reset();
                UtilStream.readStream( inStream, outStreamResult );

                String str = outStreamResult.toString();
                if ( remoteEcho )
                {
                    final int line_echo = str.indexOf( "\r\n" );
                    if ( 0 < line_echo )
                    {
                        str = str.substring( line_echo + 2 );
                    }
                }
                logger.info( ">>>" );
                logger.info( str );
                logger.info( ">>>" );
                if ( 0 <= str.indexOf( "Password:" ) )
                {
                    remoteEcho = true;
                    try
                    {
                        outStream.write( account.getPasswordAdmin().getBytes() );
                        outStream.write( "\r\n".getBytes() );
                        outStream.flush();
                    }
                    catch ( IOException e )
                    {
                        e.printStackTrace( System.err );
                    }
                }
            }

            {
                outStreamResult.reset();
                UtilStream.readStream( inStream, outStreamResult );

                String str = outStreamResult.toString();
                if ( remoteEcho )
                {
                    final int line_echo = str.indexOf( "\r\n" );
                    if ( 0 < line_echo )
                    {
                        str = str.substring( line_echo + 2 );
                    }
                }
                logger.info( ">>>" );
                logger.info( str );
                logger.info( ">>>" );
            }

        }
    }


    public void getConfigList(final Account account )
    {
        if ( null == account )
        {
            throw new IllegalArgumentException();
        }

        if ( null == account.hostname )
        {
            throw new IllegalArgumentException();
        }
        if ( 0 == account.hostname.length() )
        {
            throw new IllegalArgumentException();
        }

        JSch client = new JSch();
        this.setOption( client );

        Session session = this.connect( client, account );

        if ( null != session )
        {
            ByteArrayOutputStream outStreamResult = new ByteArrayOutputStream( 8*1024 );

            boolean remoteEcho = false;
            this.login( session, account, outStreamResult );

            Channel channel = null;
            try
            {
                channel = session.openChannel( "shell" );
            }
            catch ( JSchException e )
            {
                e.printStackTrace( System.err );
            }

            if ( null != channel )
            {
                InputStream inStream = null;
                OutputStream outStream = null;
                try
                {
                    inStream = channel.getInputStream();
                    outStream = channel.getOutputStream();
                }
                catch ( IOException e )
                {
                    e.printStackTrace( System.err );
                }

                try
                {
                    channel.connect( 3*1000 );
                }
                catch ( JSchException e )
                {
                    e.printStackTrace( System.err );
                }

                {
                    outStreamResult.reset();
                    UtilStream.readStream( inStream, outStreamResult );

                    final String str = outStreamResult.toString();
                    logger.info( ">>>" );
                    logger.info( str );
                    logger.info( ">>>" );

                    final String[] lines = str.split( "\r\n" );
                    if ( null != lines )
                    {
                        final int lines_count = lines.length;
                        if ( 1 <= lines_count )
                        {
                            final String line = lines[lines_count-1];
                            if ( null != line )
                            {
                                if ( 0 <= line.indexOf( "> " ) )
                                {
                                    remoteEcho = true;
                                    try
                                    {
                                        outStream.write( "show config list".getBytes() );
                                        outStream.write( "\r".getBytes() );
                                        outStream.flush();
                                    }
                                    catch ( IOException e )
                                    {
                                        e.printStackTrace( System.err );
                                    }
                                }
                            }
                        }
                    }
                }

                {
                    outStreamResult.reset();
                    UtilStream.readStream( inStream, outStreamResult );

                    String str = outStreamResult.toString();
                    if ( remoteEcho )
                    {
                        final int line_echo = str.indexOf( "\r\n" );
                        if ( 0 < line_echo )
                        {
                            str = str.substring( line_echo + 2 );
                        }
                    }
                    logger.info( ">>>" );
                    logger.info( str );
                    logger.info( ">>>" );
                }

            }

            //dumpOptionState( client );

            if ( null != channel )
            {
                channel.disconnect();
                channel = null;
            }
        }

        if ( null != session )
        {
            session.disconnect();
            session = null;
        }

        client = null;
    }


    public void setDefaultConfig(final Account account, final String configNo )
    {
        if ( null == account )
        {
            throw new IllegalArgumentException();
        }

        if ( null == account.hostname )
        {
            throw new IllegalArgumentException();
        }
        if ( 0 == account.hostname.length() )
        {
            throw new IllegalArgumentException();
        }

        JSch client = new JSch();
        this.setOption( client );

        Session session = this.connect( client, account );

        if ( null != session )
        {
            ByteArrayOutputStream outStreamResult = new ByteArrayOutputStream( 8*1024 );

            boolean remoteEcho = false;
            this.login( session, account, outStreamResult );

            Channel channel = null;
            try
            {
                channel = session.openChannel( "shell" );
            }
            catch ( JSchException e )
            {
                e.printStackTrace( System.err );
            }

            if ( null != channel )
            {
                InputStream inStream = null;
                OutputStream outStream = null;
                try
                {
                    inStream = channel.getInputStream();
                    outStream = channel.getOutputStream();
                }
                catch ( IOException e )
                {
                    e.printStackTrace( System.err );
                }

                try
                {
                    channel.connect( 3*1000 );
                }
                catch ( JSchException e )
                {
                    e.printStackTrace( System.err );
                }

                {
                    outStreamResult.reset();
                    UtilStream.readStream( inStream, outStreamResult );

                    final String str = outStreamResult.toString();
                    logger.info( ">>>" );
                    logger.info( str );
                    logger.info( ">>>" );
                }

                this.administrator( session, channel, account, outStreamResult );

                {
                    //outStreamResult.reset();
                    //UtilStream.readStream( inStream, outStreamResult );

                    final String str = outStreamResult.toString();
                    //logger.info( ">>>" );
                    //logger.info( str );
                    //logger.info( ">>>" );

                    final String[] lines = str.split( "\r\n" );
                    if ( null != lines )
                    {
                        final int lines_count = lines.length;
                        if ( 1 <= lines_count )
                        {
                            final String line = lines[lines_count-1];
                            if ( null != line )
                            {
                                if ( 0 <= line.indexOf( "# " ) )
                                {
                                    remoteEcho = true;
                                    try
                                    {
                                        outStream.write( "set-default-config ".getBytes() );
//                                        outStream.write( configNo.getBytes() );
                                        outStream.write( "\r\n".getBytes() );
                                        outStream.flush();
                                    }
                                    catch ( IOException e )
                                    {
                                        e.printStackTrace( System.err );
                                    }
                                }
                            }
                        }
                    }
                }

                {
                    outStreamResult.reset();
                    UtilStream.readStream( inStream, outStreamResult );

                    String str = outStreamResult.toString();
                    if ( remoteEcho )
                    {
                        final int line_echo = str.indexOf( "\r\n" );
                        if ( 0 < line_echo )
                        {
                            str = str.substring( line_echo + 2 );
                        }
                    }
                    logger.info( ">>>" );
                    logger.info( str );
                    logger.info( ">>>" );
                }

            }

            //dumpOptionState( client );

            if ( null != channel )
            {
                channel.disconnect();
                channel = null;
            }
        }

        if ( null != session )
        {
            session.disconnect();
            session = null;
        }

        client = null;
    }
}
