/**
 * @author diverKon <diverKon@gmail.com>
 * @license http://www.opensource.org/licenses/bsd-license.php
 */
package jp.ne.sakura.kkkon.yamahart;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ConnectException;
import java.net.SocketException;

import org.apache.commons.net.telnet.EchoOptionHandler;
import org.apache.commons.net.telnet.InvalidTelnetOptionException;
import org.apache.commons.net.telnet.SuppressGAOptionHandler;
import org.apache.commons.net.telnet.TelnetClient;
import org.apache.commons.net.telnet.TelnetNotificationHandler;
import org.apache.commons.net.telnet.TelnetOption;
import org.apache.commons.net.telnet.TerminalTypeOptionHandler;

/**
 *
 */
public class Telnet
{
    private static final Logger logger = LoggerFactory.getLogger( Telnet.class );

    public static class notificationHandler implements TelnetNotificationHandler
    {
        public void receivedNegotiation( final int negotiation_code, final int option_code)
        {
            String command = null;
            if ( negotiation_code == TelnetNotificationHandler.RECEIVED_DO )
            {
                command = "DO";
            }
            else if ( negotiation_code == TelnetNotificationHandler.RECEIVED_DONT )
            {
                command = "DONT";
            }
            else if ( negotiation_code == TelnetNotificationHandler.RECEIVED_WILL )
            {
                command = "WILL";
            }
            else if ( negotiation_code == TelnetNotificationHandler.RECEIVED_WONT )
            {
                command = "WONT";
            }
            final String option = TelnetOption.getOption(option_code);
            logger.info( "  Recv " + command + " for option " + option + "(" + option_code + ")" );
        }
    }

    protected void setOption( final TelnetClient client )
    {
        if ( null == client )
        {
            throw new IllegalArgumentException();
        }

        TerminalTypeOptionHandler opt_termtype = new TerminalTypeOptionHandler( "VT100" );
        opt_termtype.setInitLocal( false );
        opt_termtype.setInitRemote( false );
        opt_termtype.setAcceptLocal( true );
        opt_termtype.setAcceptRemote( false );
        EchoOptionHandler opt_echo = new EchoOptionHandler();
        opt_echo.setInitLocal( false );
        opt_echo.setInitRemote( false );
        opt_echo.setAcceptLocal( false );
        opt_echo.setAcceptRemote( true );
        SuppressGAOptionHandler opt_ga = new SuppressGAOptionHandler();
        opt_ga.setInitLocal( true );
        opt_ga.setInitRemote( true );
        opt_ga.setAcceptLocal( true );
        opt_ga.setAcceptRemote( true );

        try
        {
            client.addOptionHandler( opt_termtype );
            client.addOptionHandler( opt_echo );
            client.addOptionHandler( opt_ga );
        }
        catch ( InvalidTelnetOptionException e )
        {
            e.printStackTrace( System.err );
        }
        catch ( IOException e )
        {
            e.printStackTrace( System.err );
        }

    }

    protected void connect( final TelnetClient client, final Account account )
    {
        if ( null == client || null == account )
        {
            throw new IllegalArgumentException();
        }

        try
        {
            client.connect( account.hostname );
        }
        catch ( ConnectException e )
        {
            e.printStackTrace( System.err );
        }
        catch ( SocketException e )
        {
            e.printStackTrace( System.err );
        }
        catch ( IOException e )
        {
            e.printStackTrace( System.err );
        }

        if ( client.isConnected() )
        {
            try
            {
                Thread.sleep( 1*1000 );
            }
            catch ( InterruptedException e )
            {
                e.printStackTrace( System.err );
            }
        }

    }

    protected void login( final TelnetClient client, final Account account, final ByteArrayOutputStream outStreamResult )
    {
        if ( client.isConnected() )
        {

            InputStream inStream = client.getInputStream();
            OutputStream outStream = client.getOutputStream();

            boolean remoteEcho = false;
            // login_type=0: anonymous no-password(cold start or factory default)
            // login_type=1: username and login-password, or anonymous and login-password
            for ( int login_type = 0; login_type < 3; )
            {
                outStreamResult.reset();
                UtilStream.readStream( inStream, outStreamResult );

                String str = outStreamResult.toString();
                if ( remoteEcho )
                {
                    final int line_echo = str.indexOf( "\r\n" );
                    if ( 0 < line_echo )
                    {
                        str = str.substring( line_echo + 2 );
                    }
                }
                logger.info( ">>>" );
                logger.info( str );
                logger.info( ">>>" );

                if ( 0 <= str.indexOf( "Password:" ) )
                {
                    String password = "";
                    switch ( login_type )
                    {
                    case 0:
                        password = "";
                        break;
                    case 1:
                        password = account.getPasswordLogin();
                        break;
                    }

                    remoteEcho = false;
                    try
                    {
                        outStream.write( password.getBytes() );
                        outStream.write( "\r\n".getBytes() );
                        outStream.flush();
                    }
                    catch ( IOException e )
                    {
                        e.printStackTrace( System.err );
                    }
                    login_type += 1;
                }
                else if ( 0 <= str.indexOf("Username:") )
                {
                    remoteEcho = true;
                    try
                    {
                        outStream.write( account.getUsername().getBytes() );
                        outStream.write( "\r\n".getBytes() );
                        outStream.flush();
                    }
                    catch ( IOException e )
                    {
                        e.printStackTrace( System.err );
                    }
                }
                else if ( 0 <= str.indexOf( "Error:" ) )
                {
                    
                }
                else
                {
                    break;
                }
            }
        }
    }

    protected void administrator( final TelnetClient client, final Account account, final ByteArrayOutputStream outStreamResult )
    {
        if ( client.isConnected() )
        {
            InputStream inStream = client.getInputStream();
            OutputStream outStream = client.getOutputStream();

            boolean remoteEcho = false;
            remoteEcho = true;
            try
            {
                outStream.write( "administrator".getBytes() );
                outStream.write( "\r\n".getBytes() );
                outStream.flush();
            }
            catch ( IOException e )
            {
                e.printStackTrace( System.err );
            }

            // login_type=0: anonymous no-password(cold start or factory default)
            // login_type=1: username and login-password, or anonymous and login-password
            for ( int login_type = 0; login_type < 3; )
            {
                outStreamResult.reset();
                UtilStream.readStream( inStream, outStreamResult );

                String str = outStreamResult.toString();
                if ( remoteEcho )
                {
                    final int line_echo = str.indexOf( "\r\n" );
                    if ( 0 < line_echo )
                    {
                        str = str.substring( line_echo + 2 );
                    }
                }
                logger.info( ">>>" );
                logger.info( str );
                logger.info( ">>>" );

                if ( 0 <= str.indexOf( "Password:" ) )
                {
                    String password = "";
                    switch ( login_type )
                    {
                    case 0:
                        password = "";
                        break;
                    case 1:
                        password = account.getPasswordAdmin();
                        break;
                    }

                    remoteEcho = false;
                    try
                    {
                        outStream.write( password.getBytes() );
                        outStream.write( "\r\n".getBytes() );
                        outStream.flush();
                    }
                    catch ( IOException e )
                    {
                        e.printStackTrace( System.err );
                    }
                    login_type += 1;
                }
                else if ( 0 <= str.indexOf("Username:") )
                {
                    remoteEcho = true;
                    try
                    {
                        outStream.write( account.getUsername().getBytes() );
                        outStream.write( "\r\n".getBytes() );
                        outStream.flush();
                    }
                    catch ( IOException e )
                    {
                        e.printStackTrace( System.err );
                    }
                }
                else if ( 0 <= str.indexOf( "Error:" ) )
                {
                    
                }
                else
                {
                    break;
                }
            }
        }
    }

    protected void dumpOptionState( final TelnetClient client )
    {
        {
            for ( int i = 0; i < TelnetOption.ENCRYPTION; ++i )
            {
                final String option = TelnetOption.getOption( i );
                final boolean local = client.getLocalOptionState( i );
                final boolean remote = client.getRemoteOptionState( i );
                System.out.println( option + "(" + i + ")" + ":" + "local=" + local + ",remote=" + remote );
            }
        }
    }

    public void getConfigList(final Account account )
    {
        if ( null == account )
        {
            throw new IllegalArgumentException();
        }

        if ( null == account.hostname )
        {
            throw new IllegalArgumentException();
        }
        if ( 0 == account.hostname.length() )
        {
            throw new IllegalArgumentException();
        }

        TelnetClient client = new TelnetClient();
        //client.registerNotifHandler( new notificationHandler() );
        this.setOption( client );

        this.connect( client, account );

        if ( client.isConnected() )
        {
            InputStream inStream = client.getInputStream();
            OutputStream outStream = client.getOutputStream();

            ByteArrayOutputStream outStreamResult = new ByteArrayOutputStream( 8*1024 );

            boolean remoteEcho = false;
            this.login( client, account, outStreamResult );

            {
                //outStreamResult.reset();
                //UtilStream.readStream( inStream, outStreamResult );

                final String str = outStreamResult.toString();
                //System.out.println( ">>>" );
                //System.out.println( str );
                //System.out.println( ">>>" );

                final String[] lines = str.split( "\r\n" );
                if ( null != lines )
                {
                    final int lines_count = lines.length;
                    if ( 1 <= lines_count )
                    {
                        final String line = lines[lines_count-1];
                        if ( null != line )
                        {
                            if ( 0 <= line.indexOf( "> " ) )
                            {
                                remoteEcho = true;
                                try
                                {
                                    outStream.write( "show config list".getBytes() );
                                    outStream.write( "\r\n".getBytes() );
                                    outStream.flush();
                                }
                                catch ( IOException e )
                                {
                                    e.printStackTrace( System.err );
                                }
                            }
                        }
                    }
                }
            }

            {
                outStreamResult.reset();
                UtilStream.readStream( inStream, outStreamResult );

                String str = outStreamResult.toString();
                if ( remoteEcho )
                {
                    final int line_echo = str.indexOf( "\r\n" );
                    if ( 0 < line_echo )
                    {
                        str = str.substring( line_echo + 2 );
                    }
                }
                logger.info( ">>>" );
                logger.info( str );
                logger.info( ">>>" );
            }

            //dumpOptionState( client );

        }

        try
        {
            client.disconnect();
        }
        catch ( IOException e )
        {
            e.printStackTrace( System.err );
        }

    }


    public void setDefaultConfig(final Account account, final String configNo )
    {
        if ( null == account )
        {
            throw new IllegalArgumentException();
        }

        if ( null == account.hostname )
        {
            throw new IllegalArgumentException();
        }
        if ( 0 == account.hostname.length() )
        {
            throw new IllegalArgumentException();
        }

        TelnetClient client = new TelnetClient();
        //client.registerNotifHandler( new notificationHandler() );
        this.setOption( client );

        this.connect( client, account );

        if ( client.isConnected() )
        {
            InputStream inStream = client.getInputStream();
            OutputStream outStream = client.getOutputStream();

            ByteArrayOutputStream outStreamResult = new ByteArrayOutputStream( 8*1024 );

            boolean remoteEcho = false;
            this.login( client, account, outStreamResult );

            this.administrator( client, account, outStreamResult );

            {
                //outStreamResult.reset();
                //UtilStream.readStream( inStream, outStreamResult );

                final String str = outStreamResult.toString();
                //System.out.println( ">>>" );
                //System.out.println( str );
                //System.out.println( ">>>" );

                final String[] lines = str.split( "\r\n" );
                if ( null != lines )
                {
                    final int lines_count = lines.length;
                    if ( 1 <= lines_count )
                    {
                        final String line = lines[lines_count-1];
                        if ( null != line )
                        {
                            if ( 0 <= line.indexOf( "# " ) )
                            {
                                remoteEcho = true;
                                try
                                {
                                    outStream.write( "set-default-config ".getBytes() );
                                    outStream.write( configNo.getBytes() );
                                    outStream.write( "\r\n".getBytes() );
                                    outStream.flush();
                                }
                                catch ( IOException e )
                                {
                                    e.printStackTrace( System.err );
                                }
                            }
                        }
                    }
                }
            }

            {
                outStreamResult.reset();
                UtilStream.readStream( inStream, outStreamResult );

                String str = outStreamResult.toString();
                if ( remoteEcho )
                {
                    final int line_echo = str.indexOf( "\r\n" );
                    if ( 0 < line_echo )
                    {
                        str = str.substring( line_echo + 2 );
                    }
                }
                logger.info( ">>>" );
                logger.info( str );
                logger.info( ">>>" );

            }

            //dumpOptionState( client );

        }

        try
        {
            client.disconnect();
        }
        catch ( IOException e )
        {
            e.printStackTrace( System.err );
        }

    }

}
